-- MIT License
-- 
-- Copyright (c) 2020 Davide Berardi <berardi.dav@gmail.com>
-- 
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
-- 
-- The above copyright notice and this permission notice shall be included in all
-- copies or substantial portions of the Software.
-- 
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-- SOFTWARE.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity led_blinker is
	port (
		i_clock       : in std_logic;
		o_led_drive   : out std_logic
	     );
end led_blinker;

architecture rtl of led_blinker is
	signal o_led_status : std_logic := '0';
	signal c_24MHz      : std_logic := '0';
	signal c_100MHz_div : unsigned (11 downto 0) := 0;
	signal c_24MHz_ctr  : unsigned (14 downto 0) := 0;

	begin
		o_led_drive <= o_led_status;

		p_clock_divisor : process(i_clock) is
		begin
			if rising_edge(i_clock) then
				c_100MHz_div <= c_100MHz_div + 1;
				if c_100MHz_div = 4095 then
					c_100MHz_div <= 0;
					c_24MHz <= not c_24MHz;
				end if;
			end if;
		end process p_clock_divisor;

		-- Main process
		p_main : process (c_24MHz) is
		begin
			if rising_edge(c_24MHz) then
				c_24MHz_ctr <= c_24MHz_ctr + 1;
				if c_24MHz_ctr = 12206 then
					o_led_status <= not o_led_status;
					c_24MHz_ctr <= 0;
				end if;
			end if;
		end process p_main;
end rtl;

